<?php
/*
 * This file is part of Robot-in for ECCUBE3 Plugin
 *
 * Copyright (C) 2015 Hunglead,Inc. doishun All Rights Reserved.
 *
 * http://www.hunglead.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

namespace Plugin\HungleadRobotIn\Dao;

use Plugin\HungleadRobotIn\Dao\AbstractDao;
use Plugin\HungleadRobotIn\Utils\DateUtil;
use Plugin\HungleadRobotIn\Form\Type\ChangeStatusType;
use Doctrine\ORM\NoResultException as NoResultException;
use Doctrine\ORM\NonUniqueResultException as NonUniqueResultException;


/**
 * 受注情報用データアクセスオブジェクト
 * @author hunglead.doishun
 */
class OrderDao extends AbstractDao
{

    /**
     * 受注情報を取得
     *  # get_orders
     * 
     * @author doishun
     * @throws Exception
     * @param  array  $id       アカウント情報
     * @param  number $status   取得先ステータス
     * @param  string $fromDate 検索期間開始日
     * @param  string $toDate   検索期間終了日
     * @return array  $results
     */
    public function getOrderInfos($id, $status, $fromDate, $toDate) {
        try {
            // 認証チェック
            $this->checkId($id);

            // 受注情報の取得
            $qb = $this->app['orm.em']->createQueryBuilder();
            $qb->select("o")
               ->from("Eccube\\Entity\\Order", "o")
               ->andWhere("o.del_flg = :del_flg")
               ->andWhere("o.OrderStatus  = :status")
               ->orderBy("o.id",  "DESC");

            // プレースホルダ
            $parameters = array();
            $parameters['del_flg'] = 0;
            $parameters['status'] = $status ? $status : 1;


            // 追加条件：受注日開始
            if ($fromDate) {
                $qb->andWhere("o.create_date >= :create_date_start");
                $fromDate = DateUtil::createDateTime($fromDate);
                $parameters['create_date_start'] = $fromDate->format("Y-m-d");
            }

            // 追加条件：受注日終了
            if ($toDate) {
                $qb->andWhere("o.create_date <= :create_date_end");
                $toDate = DateUtil::createDateTime($toDate);
                $toDate->modify("+1 day");
                $parameters['create_date_end'] = $toDate->format("Y-m-d");
            }


            // DQL実行
            $qb->setParameters($parameters);
            $query = $qb->getQuery();
            $objOrder = $query->getResult();

            $results = array();
            foreach($objOrder as $_key => $srcOrder){
                if($srcOrder->getDelFlg() ==1) continue;

                $order_id = $srcOrder->getId();
                $TargetOrder = $this->app['eccube.repository.order']->find($order_id);
                // 配送情報の取得
                $srcShippings = $TargetOrder->getShippings();

                // Harc形式に変換
                $order = $this->exchangeOrder($srcOrder, $srcShippings);
                $order["unique_key"] = $order["order"]["order_date"].$order["order"]["order_code"];
                $results[] = $order;
            }


        // クエリの結果が0件の場合
        } catch (NoResultException $e) {
            $this->Logger->addError($e->getMessage()."\n".$e->__toString());
            throw new \Exception($e->getMessage());

        } catch (NonUniqueResultException $e) {
            $this->Logger->addError($e->getMessage()."\n".$e->__toString());
            throw new \Exception($e->getMessage());

        } catch (\Exception $e){
            $this->Logger->addError($e->getMessage()."\n".$e->__toString());
            throw $e;
        }

        return $results;
    }



    /**
     * 受注情報を取得
     *  # get_order
     * 
     * @author doishun
     * @throws Exception
     * @param array $id アカウント情報
     * @param int $orderId 受注ID
     */
    public function getOrderInfo($id, $orderId) {
        try {
            // 認証チェック
            $this->checkId($id);

            // 受注情報の取得
            $objOrder = $this->getOrder($orderId);

            // 配送情報の取得
            $TargetOrder = $this->app['eccube.repository.order']->find($orderId);
            $srcShippings = $TargetOrder->getShippings();

            // Harc形式に変換
            $order = $this->exchangeOrder($objOrder, $srcShippings);
            $order["unique_key"] = $order["order"]["order_date"].$order["order"]["order_code"];


        // クエリの結果が0件の場合
        } catch (NoResultException $e) {
            $this->Logger->addError($e->getMessage()."\n".$e->__toString());
            throw new \Exception($e->getMessage());

        } catch (NonUniqueResultException $e) {
            $this->Logger->addError($e->getMessage()."\n".$e->__toString());
            throw new \Exception($e->getMessage());

        } catch (\Exception $e){
            $this->Logger->addError($e->getMessage()."\n".$e->__toString());
            throw $e;
        }

        return $order;
    }



    /**
     * 受注情報を取得
     * 
     * @author doishun
     * @throws Exception
     * @param int $orderId 受注ID
     */
    private function getOrder($orderId){
        // 受注情報の取得
        $qb = $this->app['orm.em']->createQueryBuilder();
        $qb->select("o")
           ->from("Eccube\\Entity\\Order", "o")
           ->andWhere("o.id      = :id")
           ->andWhere("o.del_flg = :del_flg");

        // プレースホルダ
        $parameters = array();
        $parameters['id']      = $orderId;
        $parameters['del_flg'] = 0;

        // DQL実行
        $qb->setParameters($parameters);
        $query = $qb->getQuery();
        return $query->getSingleResult();
    }



    /**
     * EC-CUBE形式の受注情報をHARC形式の受注情報に変換
     *
     * @author doishun
     * @param array $srcOrder EC-CUBE形式受注情報
     * @param array $srcShippings EC-CUBE形式送り先情報
     * @return array HARC形式受注情報
     */
    private function exchangeOrder($srcOrder, $srcShippings) {

        $destOther = array();
        $destOther["order_fax"]   = ($srcOrder->getFax01() && $srcOrder->getFax02() && $srcOrder->getFax03())
                                  ? $srcOrder->getFax01()."-".$srcOrder->getFax02()."-".$srcOrder->getFax03() : null;
        $destOther["order_sex"]   = ($srcOrder->getSex()        instanceof \Eccube\Entity\Master\Sex)        ? $srcOrder->getSex()->getName() : "不明";
        $destOther["order_birth"] = ($srcOrder->getBirth()      instanceof \DateTime)                        ? $srcOrder->getBirth()->format('Y-m-d H:i:s') : null;
        $destOther["order_job"]   = ($srcOrder->getJob()        instanceof \Eccube\Entity\Master\Job)        ? $srcOrder->getJob()->getName()        : null;
        $destOther["device_type"] = ($srcOrder->getDeviceType() instanceof \Eccube\Entity\Master\DeviceType) ? $srcOrder->getDeviceType()->getName() : null;
        $destOther["update_date"] = $srcOrder->getUpdateDate()->format('Y-m-d H:i:s');
        $destOther["commit_date"] = ($srcOrder->getCommitDate() instanceof \DateTime) ? $srcOrder->getCommitDate()->format('Y-m-d H:i:s') : null;
        $destOther["memo01"]      = null; // memoがない…。
        $destOther["memo02"]      = null; // memoがない…。
        $destOther["memo03"]      = null; // memoがない…。
        $destOther["memo04"]      = null; // memoがない…。
        $destOther["memo05"]      = null; // memoがない…。
        $destOther["memo06"]      = null; // memoがない…。
        $destOther["memo07"]      = null; // memoがない…。
        $destOther["memo08"]      = null; // memoがない…。
        $destOther["memo09"]      = null; // memoがない…。
        $destOther["memo10"]      = null; // memoがない…。


        $destOrder = array();
        $destOrder["order_code"]     = $srcOrder->getId();
        $destOrder["order_date"]     = $srcOrder->getCreateDate()->format('Y-m-d H:i:s');
        $destOrder["order_status"]   = ($srcOrder->getOrderStatus() instanceof \Eccube\Entity\Master\OrderStatus) ? $srcOrder->getOrderStatus()->getName() : null;
        $destOrder["payment_method"] = $srcOrder->getPaymentMethod();
        $destOrder["payment_date"]   = $srcOrder->getPaymentDate();
        $destOrder["order_price"]    = $srcOrder->getPaymentTotal();
        $destOrder["delivery_cost"]  = $srcOrder->getDeliveryFeeTotal();
        $destOrder["commission"]     = $srcOrder->getCharge();
        $destOrder["discount"]       = $srcOrder->getDiscount();
        $destOrder["use_point"]      = 0; // ポイントは標準機能ではなくプラグインになったのでややこしい。というかプラグイン公開されないと組み込めない
        $destOrder["remarks"]        = $srcOrder->getMessage();
        $destOrder["order_tax"]      = $srcOrder->getTax();
        $destOrder["other"]          = $destOther;

        $destCustomer = array();
        $destCustomer["name"]    = $srcOrder->getName01()." ".$srcOrder->getName02();
        $destCustomer["kana"]    = $srcOrder->getKana01()." ".$srcOrder->getKana02();
        $destCustomer["zipcode"] = $srcOrder->getZip01() ."-".$srcOrder->getZip02();
        $destCustomer["address"] = $srcOrder->getPref()->getName().$srcOrder->getAddr01().$srcOrder->getAddr02();
        $destCustomer["phone"]   = $srcOrder->getTel01() ."-".$srcOrder->getTel02()."-".$srcOrder->getTel03();
        $destCustomer["email"]   = $srcOrder->getEmail();
        $destCustomer["company"] = $srcOrder->getCompanyName();


        $destDetails = array();

        $order_id = $srcOrder->getId();
        $TargetOrder = $this->app['eccube.repository.order']->find($order_id);
        // 配送情報の取得
        $srcShippings = $TargetOrder->getShippings();
        foreach ($srcShippings as $_srcShippings) {
            if($_srcShippings->getDelFlg() == 1) continue;

            // dtb_shipment_item
            $objShipimentItems = $_srcShippings->getShipmentItems();
            foreach($objShipimentItems as $_objShipimentItems){
                // 配送情報
                $destDelivery = array();
                $destDelivery["delivery_method"]    = $_srcShippings->getDelivery()->getName();
                $destDelivery["delivery_hope_date"] = ($_srcShippings->getShippingDeliveryDate() instanceof \DateTime)
                                                    ? $_srcShippings->getShippingDeliveryDate()->format('Y-m-d H:i:s') : null;
                $destDelivery["delivery_hope_time"] = $_srcShippings->getShippingDeliveryTime();
                $destDelivery["delivery_memo"]      = null;

                // 配送先情報
                $destShip = array();
                $destShip["name"]    = $_srcShippings->getName01()." ".$_srcShippings->getName02();
                $destShip["kana"]    = $_srcShippings->getKana01()." ".$_srcShippings->getKana02();
                $destShip["zipcode"] = $_srcShippings->getZip01() ."-".$_srcShippings->getZip02();
                $destShip["address"] = $_srcShippings->getPref()->getName().$_srcShippings->getAddr01().$_srcShippings->getAddr02();
                $destShip["phone"]   = $_srcShippings->getTel01()."-".$_srcShippings->getTel02()."-".$_srcShippings->getTel03();
                $destShip["company"] = $_srcShippings->getCompanyName();


                // 購入商品情報
                $destDetail = array();
                $destDetail["item_code"] = $_objShipimentItems->getProductCode();
                $destDetail["item_name"] = $_objShipimentItems->getProductName();
                if ($_objShipimentItems->getClassCategoryName1() && $_objShipimentItems->getClassCategoryName2()) {
                    $destDetail["item_select"] = $_objShipimentItems->getClassCategoryName1()."/".$_objShipimentItems->getClassCategoryName2();
                } else if ($_objShipimentItems->getClassCategoryName1()) {
                    $destDetail["item_select"] =  $_objShipimentItems->getClassCategoryName1();
                } else {
                    $destDetail["item_select"] = null;
                }
                $destDetail["unit_price"] = $_objShipimentItems->getPrice();
                $destDetail["quantity"]   = $_objShipimentItems->getQuantity();
                $destDetail["delivery"]   = $destDelivery;
                $destDetail["ship"]       = $destShip;

                // 注文情報として登録
                $destDetails[] = $destDetail;
            }
        }


        $item = array();
        $item["order"]    = $destOrder;
        $item["customer"] = $destCustomer;
        $item["details"]  = $destDetails;

        return $item;
    }



    /**
     * ステータスを変更
     *
     * @author doishun
     * @throws Exception
     * @param array $id アカウント情報
     * @param array $targetOrders 対象受注情報
     * @param number $status 移動先ステータス
     * @return array ステータス変更結果
     */
    public function changeStatus($id, $targetOrders, $status, $request) {
        $this->checkId($id);

        $results = array();

        $this->app['orm.em']->getConnection()->beginTransaction();


        foreach ($targetOrders as $order) {
            $orderId = $order["orderCode"];
            if(!$orderId) continue;

            // 受注番号の書式チェック
            if(!preg_match("/^[0-9]+$/u", $orderId)) {
                $results[$orderId] = "Invalid OrderCode.";
                continue;
            }

            try{
                // 更新対象の受注情報
                $TargetOrder = $this->app['eccube.repository.order']->find($orderId);
                if (is_null($TargetOrder)) {
                    throw new NotFoundHttpException();
                }

                // 編集前の受注情報を保持
                $OriginOrder = clone $TargetOrder;

                // 更新ステータスを定義
                $TargetOrder->setOrderStatus($this->app['eccube.repository.order_status']->find($status));

                // 更新の前後で同じ受注ステータス
                if($TargetOrder->getOrderStatus()->getId() == $OriginOrder->getOrderStatus()->getId()){
                    $results[$orderId] = "Needless To Move.";
                    continue;
                }

                // 受注日/発送日/入金日の更新.
                $this->updateDate($TargetOrder, $OriginOrder);
                $this->app['orm.em']->persist($TargetOrder);
                $results[$orderId] = "Success.";


            // クエリの結果が0件の場合
            } catch (NotFoundHttpException $e) {
                $this->Logger->addError("Not Found:{$orderId}\n".$e->__toString());
                $results[$orderId] = "Not Found.";

            } catch (\Exception $e){
                $this->Logger->addError("Not Found:{$orderId}\n".$e->__toString());
                $results[$orderId] = "Not Found.";
            }
        }

        $this->app['orm.em']->getConnection()->commit();
        $this->app['orm.em']->flush();
        $this->app['orm.em']->close();

        return $results;
    }



    /**
     * 受注ステータスに応じて, 受注日/入金日/発送日を更新する,
     * 発送済ステータスが設定された場合は, お届け先情報の発送日も更新を行う.
     *
     * 編集の場合
     * - 受注ステータスが他のステータスから発送済へ変更された場合に発送日を更新
     * - 受注ステータスが他のステータスから入金済へ変更された場合に入金日を更新
     *
     * 新規登録の場合
     * - 受注日を更新
     * - 受注ステータスが発送済に設定された場合に発送日を更新
     * - 受注ステータスが入金済に設定された場合に入金日を更新
     *
     * TODO 受注ステータスの定数化.
     *
     * @see Eccube\Controller\Admin\Order\EditController.updateDate
     * @param $TargetOrder
     * @param $OriginOrder
     */
    protected function updateDate($TargetOrder, $OriginOrder)
    {
        $dateTime = new \DateTime();

        // 編集
        if ($TargetOrder->getId()) {
            // 発送済
            if ($TargetOrder->getOrderStatus()->getId() == 5) {
                // 編集前と異なる場合のみ更新
                if ($TargetOrder->getOrderStatus()->getId() != $OriginOrder->getOrderStatus()->getId()) {
                    $TargetOrder->setCommitDate($dateTime);
                    // お届け先情報の発送日も更新する.
                    $Shippings = $TargetOrder->getShippings();
                    foreach ($Shippings as $Shipping) {
                        $Shipping->setShippingCommitDate($dateTime);
                    }
                }
                // 入金済
            } elseif ($TargetOrder->getOrderStatus()->getId() == 6) {
                // 編集前と異なる場合のみ更新
                if ($TargetOrder->getOrderStatus()->getId() != $OriginOrder->getOrderStatus()->getId()) {
                    $TargetOrder->setPaymentDate($dateTime);
                }
            }
            // 新規
        } else {
            // 発送済
            if ($TargetOrder->getOrderStatus()->getId() == 5) {
                $TargetOrder->setCommitDate($dateTime);
                // お届け先情報の発送日も更新する.
                $Shippings = $TargetOrder->getShippings();
                foreach ($Shippings as $Shipping) {
                    $Shipping->setShippingCommitDate($dateTime);
                }
                // 入金済
            } elseif ($TargetOrder->getOrderStatus()->getId() == 6) {
                $TargetOrder->setPaymentDate($dateTime);
            }
            // 受注日時
            $TargetOrder->setOrderDate($dateTime);
        }
    }


}
