<?php
/*
 * This file is part of Robot-in for ECCUBE3 Plugin
 *
 * Copyright (C) 2015 Hunglead,Inc. doishun All Rights Reserved.
 *
 * http://www.hunglead.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

namespace Plugin\HungleadRobotIn\Dao;

use Plugin\HungleadRobotIn\Dao\AbstractDao;
use Doctrine\ORM\NoResultException as NoResultException;
use Doctrine\ORM\NonUniqueResultException as NonUniqueResultException;


/**
 * 商品情報用データアクセスオブジェクト
 * @author hunglead.doishun
 *
 */
class ProductDao extends AbstractDao
{

    /**
     * 商品コードを持つ商品情報を取得
     *
     * @author doishun
     * @throws Exception
     * @param  string $productCode
     * @return array  $objProduct  \Eccube\Entity\Productの配列
     */
    function getProducts($productCode)
    {
        $objProduct = null;

        try {
            $qb = $this->app['orm.em']->createQueryBuilder();
            $qb->select("pc")
               ->from("Eccube\\Entity\\ProductClass", "pc")
               ->innerJoin('pc.Product', 'p')
               ->where("pc.del_flg = 0 and pc.code = :code and p.del_flg = 0");

            // プレースホルダ
            $parameters = array();
            $parameters['code'] = $productCode;

            // DQL実行
            $qb->setParameters($parameters);
            $query = $qb->getQuery();

            $objProductClass = $query->getResult();
            if(count($objProductClass) == 0) throw new NoResultException($productCode);

            foreach($objProductClass as $_objProductClass){
                $objProduct[] = $_objProductClass->getProduct();
            }

        // クエリの結果が0件の場合
        } catch (NoResultException $e) {
            throw $e;

        } catch (\Exception $e){
            throw $e;
        }

        return $objProduct;
    }



    /**
     * 商品情報を更新
     *
     * @author doishun
     * @param array $id アカウント情報
     * @param array $products 商品情報
     */
    public function update($id, $products) {
        $message = "";
        $err_cnt = 0;  // エラー件数
        $upd_cnt = 0;  // 更新件数

        try {
            // 認証チェック
            $this->checkId($id);

            $this->app['orm.em']->getConnection()->beginTransaction();

            // 商品コードの配列を回す
            foreach($products as $key => $val) {
                $productCode = empty($val["product_code"]) ? 0 : $val["product_code"];
                try {
                    $objProducts = $this->getProducts($productCode);

                    // 更新情報
                    $stock = $val["stock"] == -1 ? null : mb_convert_kana($val["stock"], 'n');
                    $price = $val["price"] == -1 ? null : mb_convert_kana($val["price"], 'n');
                    //$label = $val["label"];

                    $sqlval = array();
                    if (isset($stock) && $stock != "") {
                        $sqlval["stock"] = $stock;
                    }
                    if (isset($price) && $price != "") {
                        $sqlval["price02"] = $price;
                    }

                    // 入力値が空
                    if (empty($sqlval)) {
                        throw new \Exception("ERROR: 価格及び在庫数の更新情報がいずれも空白で指定されました。");
                    }

                    // 数値チェック
                    $isValid = true;
                    $arrErr  = array();
                    while(true){
                        foreach($sqlval as $_key => $_val){
                            if(strlen($_val) > 0 && preg_match("/^[0-9]+$/", $_val) !== 1){
                                $arrErr[] = "ERROR: {$_key} のnumeric型の入力構文が無効です: ".$_val;
                                $isValid  = false;
                            }
                        }
                        if(!$isValid){
                            throw new \Exception(implode(", ", $arrErr));
                        }
                        break;
                    }


                    // 更新処理
                    foreach($objProducts as $objProduct){
                        $objProductClasses = $objProduct->getProductClasses();
                        foreach($objProductClasses as $_objProductClasses){
                            // 商品コードチェック
                            if($productCode != $_objProductClasses->getCode()) continue;

                            // 価格更新
                            if(isset($sqlval["price02"])){
                                $_objProductClasses->setPrice02($sqlval["price02"]);
                                $this->app['orm.em']->persist($_objProductClasses);
                            }

                            // 在庫更新
                            if(isset($sqlval["stock"])){
                                // dtb_product_class.stock
                                $_objProductClasses->setStock($sqlval["stock"]);
                                $this->app['orm.em']->persist($_objProductClasses);

                                // dtb_product_stock.stock
                                $_objProductStock = $_objProductClasses->getProductStock();
                                $_objProductStock->setStock($sqlval["stock"]);
                                $_objProductStock->setProductClass($_objProductClasses);
                                $_objProductClasses->setProductStock($_objProductStock);
                                $this->app['orm.em']->persist($_objProductStock);
                            }

                            $upd_cnt++;
                        }
                    }


                // 対象の商品コードの商品が見つからない場合
                } catch (NoResultException $e) {
                    $err_cnt++;
                    if ($err_cnt == 1) {
                        $message = "NG\n";
                    }
                    $message .= "no_product(product_code=". $productCode. ")\n";
                    $this->Logger->addError("no_product(product_code=". $productCode. ")\n".$e->__toString());

                // 想定外のエラー発生
                } catch (\Exception $e){
                    $err_cnt++;
                    if ($err_cnt == 1) {
                        $message = "NG\n";
                    }
                    $message .= "error(product_code=". $productCode. ":".$e->getMessage().")\n";
                    $this->Logger->addError("error(product_code=". $productCode. ")\n".$e->__toString());
                }
            }

            $this->app['orm.em']->getConnection()->commit();
            $this->app['orm.em']->flush();
            $this->app['orm.em']->close();

            // エラー件数0かつ取り込み件数あり
            if($err_cnt == 0 && $upd_cnt > 0 ) $message = "OK";

            // 取り込み件数なし # foreachが回ってない
            if($err_cnt == 0 && $upd_cnt == 0) $message = "NG\n". "no_query";


        } catch (\Exception $e){
            $this->Logger->addError($e->__toString());
            throw $e;
        }


        return $message;
    }


}
